@@ -15,7 +15,7 @@ fitme_body <- function(processed,
   } else proc1 <- processed
   verbose <-  proc1$verbose
   HLnames <- (c(names(formals(HLCor)),names(formals(HLfit)),
-                names(formals(designL.from.Corr)),names(formals(make_scaled_dist))))  ## cf parallel code in HLCor.obj
+                names(formals(.spaMM.data$options$mat_sqrt_fn)),names(formals(make_scaled_dist))))  ## cf parallel code in HLCor.obj
   ## fill HLCor.args
   good_dotnames <- intersect(names(dotlist),HLnames) ## those specifically for the called fns as def'd by HLnames
   if (length(good_dotnames)) {
@@ -71,7 +71,8 @@ fitme_body <- function(processed,
   
   
   processedHL1 <- proc1$HL[1] ## there's also HLmethod in processed<[[]]>$callargs
-  needHLCor_specific_args <- (length(unlist(lower$corrPars)) || length(intersect(corr_types,c("Matern","adjacency","AR1","corrMatrix"))))
+  needHLCor_specific_args <- (length(unlist(lower$corrPars)) || 
+                                length(intersect(corr_types,c("Matern","Cauchy","adjacency","AR1","corrMatrix"))))
   if (needHLCor_specific_args) {
     HLcallfn.obj <- "HLCor.obj" 
     HLcallfn <- "HLCor"
@@ -105,11 +106,12 @@ fitme_body <- function(processed,
     if (identical(verbose["getCall"][[1L]],TRUE)) { ## toget an optim call with its initial value. Then HLcallfn is called and its call returned.
       ## confint -> get_HLCorcall needs an HLCor call with the following ranFix
       ranPars_in_refit <- structure(.modify_list(fixed,init.optim),
-                                    # I write "F I X" as a TAG for this modif type attribute:
+                                    # I label this "F I X" as a TAG for this modif type attribute:
                                     type=.modify_list(relist(rep("fix",length(unlist(fixed))),fixed), #attr(fixed,"type"), 
                                                      relist(rep("outer",length(initvec)),init.optim)) )
     } else {
       use_SEM <- (!is.null(processedHL1) && processedHL1=="SEM")
+      time2 <- Sys.time()
       if (use_SEM) {
         if (is.null(proc1$SEMargs$control_pmvnorm$maxpts)) {
           if (length(LowUp$lower)) {
@@ -120,33 +122,93 @@ fitme_body <- function(processed,
         ## and then we need to put back the code for logL from smoothing at the end of this function
         ## beware of SEM case with length(lower)=0
         refit_info <- control[["refit"]] ## may be a NULL/ NA/ boolean or a list of booleans 
-        if (is.null(refit_info) || is.na(refit_info)) refit_info <- FALSE ## alternatives are TRUE or an explicit list, but no longer NULL
+        if ( is.null(refit_info) || is.na(refit_info)) refit_info <- FALSE ## alternatives are TRUE or an explicit list or NULL
       } else {
-        optPars <- .new_locoptim(init.optim, 
+        optPars <- .new_locoptim(init.optim, ## F I X M E try to use gradient?
                                  LowUp, 
                                  control, objfn_locoptim=.objfn_locoptim, 
                                  HLcallfn.obj=HLcallfn.obj, anyHLCor_obj_args=anyHLCor_obj_args, 
                                  user_init_optim=user_init_optim)
-        refit_info <- attr(optPars,"refit_info") ## 'derives' from control[["refit"]]
+        refit_info <- attr(optPars,"refit_info") ## 'derives' from control[["refit"]] with modification ## may be NULL but not NA
       }
+      optim_time <- .timerraw(time2)
       ranPars_in_refit <- structure(.modify_list(fixed,optPars),
                                    type=.modify_list(relist(rep("fix",length(unlist(fixed))),fixed), #attr(fixed,"type"),
                                                      relist(rep("outer",length(unlist(optPars))),optPars)))
-      if ( is.list(refit_info)) {refit_phi <- refit_info$phi} else refit_phi <- refit_info ## result may be NULL in list case
-      init_refit <- list()
-      if (identical(refit_phi,TRUE) && ! is.null(optPars$trPhi)) {
-        init_refit$phi <- .dispInv(optPars$trPhi)
-        ranPars_in_refit$trPhi <- NULL
-        attr(ranPars_in_refit,"type")$trPhi <- NULL 
+      augZXy_phi_est <- proc1$augZXy_env$phi_est ## may be NULL: if phi was not estimated by augZXy
+      
+      any_nearly_singular_covmat <- FALSE
+      if (! is.null(optPars$trRanCoefs)) {
+        ranCoefs <- optPars$trRanCoefs # copy names...
+        for (char_rd in names(optPars$trRanCoefs)) {
+          ranCoefs[[char_rd]] <- .ranCoefsInv(optPars$trRanCoefs[[char_rd]])
+          covmat <- .calc_cov_from_ranCoef(ranCoefs[[char_rd]])
+          if (kappa(covmat)>1e14 || min(eigen(covmat,only.values = TRUE)$values)<1e-6) any_nearly_singular_covmat <- TRUE
+        }
       }
-      if ( is.list(refit_info)) {refit_lambda <- refit_info$lambda} else refit_lambda <- refit_info
-      if (identical(refit_lambda,TRUE) && ! is.null(optPars$trLambda)) {
-        lambdapos <- as.integer(names(optPars$trLambda))
-        lambda <- rep(NA,max(lambdapos))
-        lambda[lambdapos] <- .dispInv(optPars$trLambda)
-        init_refit$lambda <- lambda
-        ranPars_in_refit$trLambda <- NULL
-        attr(ranPars_in_refit,"type")$trLambda <- NULL 
+      init_refit <- list()
+      if ( is.null(refit_info) ) { ## not the case with SEM
+        # if (any_nearly_singular_covmat) message("Nearly-singular cov matrix => refitting random-coefficient parameters by default.")
+        if (any_nearly_singular_covmat) message("Nearly-singular cov matrix => Consider refitting with refit_info=TRUE.")
+        refit_info <- list(phi=FALSE,lambda=(! is.null(optPars$trRanCoefs)),ranCoefs=(FALSE && any_nearly_singular_covmat))
+      } else if ( is.list(refit_info) ) { ## never the default
+        if (any_nearly_singular_covmat) {
+          if (is.null(refit_info$ranCoefs)) {
+            # message("Nearly-singular cov matrix => refitting random-coefficient parameters by default.")
+          } else if ( ! refit_info$ranCoefs ) message("Nearly-singular cov matrix => Consider refitting with refit_info$ranCoefs=FALSE.")
+        } 
+        refit_info <- .modify_list( list(phi=FALSE,lambda=(! is.null(optPars$trRanCoefs)),
+                                         ranCoefs=(FALSE && any_nearly_singular_covmat)), refit_info) 
+        ## lambda=TRUE becomes the dafault (test random-slope 'ares' shows the effect)
+      } else {
+        if (any_nearly_singular_covmat) {
+          if ( ! refit_info) message("Nearly-singular cov matrix => Consider refitting with refit_info=TRUE.")
+        } 
+        refit_info <- list(phi=refit_info,lambda=refit_info,ranCoefs=refit_info) # default with SEM is all FALSE
+      }
+      # refit:
+      if (identical(refit_info$phi,TRUE)) {
+        if ( ! is.null(optPars$trPhi)) {
+          init_refit$phi <- .dispInv(optPars$trPhi)
+          ranPars_in_refit$trPhi <- NULL
+          attr(ranPars_in_refit,"type")$trPhi <- NULL 
+        } else if (proc1$augZXy_cond ) {
+          init_refit$phi <- augZXy_phi_est # might still be NULL...
+        }
+      }
+      # refit, or rescale by augZXy_phi_est even if no refit:
+      if ( ! is.null(augZXy_phi_est) || identical(refit_info$lambda,TRUE)) {
+        if (! is.null(optPars$trLambda)) { # does NOT include the lambdas of the ranCoefs
+          lambdapos <- as.integer(names(optPars$trLambda))
+          lambda <- rep(NA,max(lambdapos))
+          names(lambda) <- paste(seq_len(length(lambda)))
+          lambda[lambdapos] <- .dispInv(optPars$trLambda)
+          if ( ! is.null(augZXy_phi_est)) { lambda <- lambda * augZXy_phi_est }
+          if (identical(refit_info$lambda,TRUE)) {
+            init_refit$lambda <- lambda
+            ranPars_in_refit$trLambda <- NULL
+            attr(ranPars_in_refit,"type")$trLambda <- NULL
+          } else ranPars_in_refit$trLambda <- .dispFn(lambda) ## rescale, but no refit
+        }
+      }
+      ## refit, or rescale by augZXy_phi_est even if no refit:
+      if ( ! is.null(augZXy_phi_est)  || identical(refit_info$ranCoefs,TRUE)) {
+        if (! is.null(optPars$trRanCoefs)) {
+          for (char_rd in names(ranCoefs)) {
+            rancoef <- ranCoefs[[char_rd]]
+            if ( ! is.null(augZXy_phi_est)) {
+              Xi_ncol <- attr(rancoef,"Xi_ncol")
+              lampos <- rev(length(rancoef) -cumsum(seq(Xi_ncol))+1L)  ## NOT cumsum(seq(Xi_cols))
+              rancoef[lampos] <- rancoef[lampos] *augZXy_phi_est
+              rancoef <- as.vector(rancoef) ## as.vector drops attributes
+            } 
+            if (identical(refit_info$ranCoefs,TRUE)) { 
+              init_refit$ranCoefs[[char_rd]] <- rancoef
+              ranPars_in_refit$trRanCoefs[char_rd] <- NULL
+              attr(ranPars_in_refit,"type")$trRanCoefs[char_rd] <- NULL
+            } else ranPars_in_refit$trRanCoefs[[char_rd]] <- .ranCoefsFn(rancoef)  
+          }
+        }      
       }
       if (length(init_refit)) HLCor.args$init.HLfit <- .modify_list(HLCor.args$init.HLfit, init_refit)  
     } ## end if ...getCall... else
@@ -154,14 +216,17 @@ fitme_body <- function(processed,
     # refit_info is list if so provided by user, else typically boolean. An input NA should have been converted to something else (not documented).
     if (needHLCor_specific_args) {
       attr(ranPars_in_refit,"moreargs") <- moreargs 
-      HLCor.args$ranPars <- ranPars_in_refit ## variable locally
-    } else HLCor.args$ranFix <- ranPars_in_refit ## variable locally  
-    
+      HLCor.args$ranPars <- ranPars_in_refit 
+    } else HLCor.args$ranFix <- ranPars_in_refit 
+  } else if (len_ranPars <- length(unlist(HLCor.args$ranPars))){ ## Set attribute
+    HLCor.args$ranPars <- structure(HLCor.args$ranPars,
+                                  type = relist(rep("fix", len_ranPars), HLCor.args$ranPars))
   }
   #
   # if processed is an envir, the following is not local to anyHLCor_obj_args$processed but change processed globally
   .assignWrapper(HLCor.args$processed,"return_only <- NULL") 
   .assignWrapper(HLCor.args$processed,"verbose['warn'] <- TRUE") ## important!
+  #browser()
   hlcor <- do.call(HLcallfn,HLCor.args) ## recomputation post optimization, or only computation if length(initvec)=0
   if (is.call(hlcor)) {
     ## then do.call(HLcallfn,HLCor.args) has retuned the call, not the fit. 
@@ -171,7 +236,8 @@ fitme_body <- function(processed,
   # hlcor may have received attr(.,"info.uniqueGeo") from HLCor_body.
   if (length(initvec)) {
     attr(hlcor,"optimInfo") <- list(LUarglist=LUarglist, optim.pars=optPars, 
-                                    objective=proc1$objective) ## processed was erased for safety
+                                    objective=proc1$objective,
+                                    optim_time=optim_time) ## processed was erased for safety
   }
   ## substantial effect on object size! :
   lsv <- c("lsv",ls())
