@@ -6,9 +6,11 @@ data("scotlip")
 
 ## same without optim: run in scotlip examples; cf also autoregressive.Rd for ML fits
 
-set.seed(124)
-eigenv <- sym_eigen(Nmatrix) ## could use eigen(,symmetric=TRUE)
-Lmat <- eigenv$u %*% diag(sqrt(1/(1-0.17*eigenv$d)))
+#set.seed(124)
+#eigenv <- sym_eigen(Nmatrix) ## could use eigen(,symmetric=TRUE)
+set.seed(129) ## mayny samples will diverge (ML on binary response) or undemonstratively fit for extreme rho values
+eigenv <- eigen(Nmatrix, symmetric=TRUE) 
+Lmat <- eigenv$vectors %*% diag(sqrt(1/(1-0.17*eigenv$values)))
 lp <- 0.1 + 3* Lmat %*% rnorm(ncol(Lmat)) ## single intercept beta =0.1; lambda=3
 resp <- rbinom(ncol(Lmat),1,1/(1+exp(-lp)))
 donn <- data.frame(npos=resp,nneg=1-resp,gridcode=scotlip$gridcode)
@@ -17,7 +19,7 @@ if (FALSE) { ## HLCor/corrHlfit already compared on scotlip by test-spaMM.R
   # corrHLfit without corners was poor here
   # CAR by Laplace with 'outer' estimation of rho
   # *** fitme is not very convicing, stops early ***
-  blob <- fitme(cbind(npos,nneg)~1 +adjacency(1|gridcode),
+  blob <- fitme(bind(npos,nneg)~1 +adjacency(1|gridcode),
                     adjMatrix=Nmatrix,family=binomial(probit),data=donn,method="ML",control.HLfit = list(LevenbergM=FALSE)) 
   AIC(blob)
 }
