@@ -1,4 +1,32 @@
 cat("\ntest of prediction variance:")
+
+## two ranefs with estimated lambdas:
+data("blackcap")
+set.seed(123)
+somegrp <- cbind(blackcap,grp=sample(2,14,replace=TRUE))
+#somegrp <- cbind(blackcap,grp=c(rep(1,7),rep(2,7))) ## to test cov mat with nonzero var of grp effect
+tworanefs <- corrHLfit(migStatus ~ 1 +  (1|grp) +Matern(1|latitude+longitude),data=somegrp,
+                       ranFix=list(nu=4,rho=0.4,phi=0.05))
+expected <- c("Gibraltar"=0.04880579, "CapeVerde"=0.04884620, "SouthernFrance"=0.04197530, 
+              "LaPalma"=0.03337928, "Madeira"=0.04360190)
+p1 <- get_predVar(tworanefs)[1:5]
+testthat::expect_true(diff(range(p1 -expected))<1e-8)
+p1 <- get_predVar(tworanefs,newdata=somegrp[1:5,])
+testthat::expect_true(diff(range(p1 -expected))<1e-8)
+ranefstwo <- corrHLfit(migStatus ~ 1  +Matern(1|latitude+longitude)+  (1|grp),data=somegrp,
+                       ranFix=list(nu=4,rho=0.4,phi=0.05))
+p2 <- get_predVar(ranefstwo,newdata=somegrp[1:5,])
+testthat::expect_true(diff(range(p1-p2))<1e-10)
+
+twolambda <- corrHLfit(migStatus ~ 1 +  Matern(1|latitude+longitude) +Matern(1|latitude+longitude),data=blackcap,
+                       ranFix=list(corrPars=list("1"=list(nu=4,rho=0.4),"2"=list(nu=4,rho=0.4)),phi=0.05))
+## but in that case the exact loglamInfo is exactly singular! 
+## (this makes sense: ones cannot separately estimate the two lambda's)
+## any attempt at computing a non-singular loglamInfo will amplify floating point error.
+onelambda <- corrHLfit(migStatus ~ 1 + Matern(1|latitude+longitude),data=blackcap,
+                       ranFix=list(nu=4,rho=0.4,phi=0.05))
+testthat::expect_true(diff(range(get_predVar(twolambda)[1:5]-get_predVar(onelambda)[1:5]))<1e-7)
+
 # examples from Booth & Hobert 1998 JASA
 
 npos <- c(11,16,14,2,6,1,1,4,10,22,7,1,0,0,1,6)
@@ -23,6 +51,20 @@ if(requireNamespace("rsae", quietly = TRUE)) {
   testthat::expect_equal(res[12],c("32"=27.80684),tolerance=2e-4)
   res <- get_predVar(fitobject,newdata=newXandZ,variances=list(disp=FALSE)) ## sum of cols nu and beta in Table 7
   testthat::expect_equal(res[12],c("32"=26.56215),tolerance=2e-4)
+  
+  ## comparison dense vs. spprec with use_spprec_qr on an REML non-spatial fit 
+  # (bc spprec does not handle Matern and use_spprec_qr affects only REML)
+  oldop <- spaMM.options(sparse_precision=FALSE)
+  fitdense <- HLfit(HACorn ~ PixelsCorn + PixelsSoybeans + (1|CountyName),data=landsat[-33,],HLmethod="REML")
+  spaMM.options(sparse_precision=TRUE)
+  oldop2 <- spaMM.options(use_spprec_QR=TRUE)
+  # trace(spaMM:::.calc_spprec_hatval_ZX_by_QR,tracer=quote(print(".calc_spprec_hatval_ZX_by_QR")))
+  # trace(spaMM:::.calc_Md2hdvb2_info_spprec_by_QR,tracer=quote(print(".calc_Md2hdvb2_info_spprec_by_QR")))
+  fitsparse <- HLfit(HACorn ~ PixelsCorn + PixelsSoybeans + (1|CountyName),data=landsat[-33,],HLmethod="REML")
+  spaMM.options(oldop); spaMM.options(oldop2)
+  testthat::expect_true(diff(range(get_predVar(fitdense,newdata=newXandZ)-get_predVar(fitsparse,newdata=newXandZ)))<1e-8)
+  testthat::expect_true(diff(range(get_predVar(fitdense,newdata=newXandZ,variances=list(disp=FALSE)) - 
+                                   get_predVar(fitsparse,newdata=newXandZ,variances=list(disp=FALSE))))<1e-8)
 } else {
   cat( "package 'rsae' not available, cannot run prediction variance test.\n" )
 }
